@@ -14,13 +14,13 @@ import (
 func testGetWantedActionMetadata() *ActionMetadata {
 	want := &ActionMetadata{
 		Name: "My action",
-		Inputs: map[string]ActionMetadataInputRequired{
-			"name":     false,
-			"message":  true,
-			"addition": false,
+		Inputs: ActionMetadataInputs{
+			"name":     {"name", false},
+			"message":  {"message", true},
+			"addition": {"addition", false},
 		},
-		Outputs: map[string]struct{}{
-			"user_id": {},
+		Outputs: ActionMetadataOutputs{
+			"user_id": {"user_id"},
 		},
 	}
 	return want
@@ -76,6 +76,24 @@ func TestLocalActionsFindMetadata(t *testing.T) {
 			}
 		}
 	})
+
+	t.Run("./uppercase", func(t *testing.T) {
+		want := testGetWantedActionMetadata()
+		for _, i := range want.Inputs {
+			i.Name = strings.ToUpper(i.Name)
+		}
+		for _, o := range want.Outputs {
+			o.Name = strings.ToUpper(o.Name)
+		}
+
+		have, err := c.FindMetadata("./uppercase")
+		if err != nil {
+			t.Fatal(err)
+		}
+		if !cmp.Equal(want, have) {
+			t.Fatal(cmp.Diff(want, have))
+		}
+	})
 }
 
 func TestLocalActionsFindConcurrently(t *testing.T) {
@@ -238,6 +256,36 @@ func TestLocalActionsBrokenMetadata(t *testing.T) {
 	}
 }
 
+func TestLocalActionsDuplicateInputsOutputs(t *testing.T) {
+	proj := &Project{filepath.Join("testdata", "action_metadata"), nil}
+	c := NewLocalActionsCache(proj, nil)
+
+	for _, tc := range []struct {
+		spec string
+		want string
+	}{
+		{
+			spec: "./input-duplicate",
+			want: "input \"FOO\" is duplicated",
+		},
+		{
+			spec: "./output-duplicate",
+			want: "output \"FOO\" is duplicated",
+		},
+	} {
+		t.Run(tc.spec, func(t *testing.T) {
+			m, err := c.FindMetadata(tc.spec)
+			if err == nil {
+				t.Fatal("error was not returned", m)
+			}
+			msg := err.Error()
+			if !strings.Contains(msg, tc.want) {
+				t.Fatalf("error %q was expected to include %q", msg, tc.want)
+			}
+		})
+	}
+}
+
 func TestLocalActionsConcurrentFailures(t *testing.T) {
 	n := 10
 	proj := &Project{filepath.Join("testdata", "action_metadata"), nil}
@@ -408,12 +456,12 @@ inputs:
     required: true`,
 			want: ActionMetadata{
 				Name: "Test",
-				Inputs: map[string]ActionMetadataInputRequired{
-					"input1": false,
-					"input2": false,
-					"input3": false,
-					"input4": false,
-					"input5": true,
+				Inputs: ActionMetadataInputs{
+					"input1": {"input1", false},
+					"input2": {"input2", false},
+					"input3": {"input3", false},
+					"input4": {"input4", false},
+					"input5": {"input5", true},
 				},
 			},
 		},
@@ -428,9 +476,9 @@ outputs:
 `,
 			want: ActionMetadata{
 				Name: "Test",
-				Outputs: map[string]struct{}{
-					"output1": {},
-					"output2": {},
+				Outputs: ActionMetadataOutputs{
+					"output1": {"output1"},
+					"output2": {"output2"},
 				},
 			},
 		},
@@ -459,7 +507,7 @@ func TestActionMetadataYAMLUnmarshalError(t *testing.T) {
 			what: "invalid inputs",
 			input: `name: Test
 inputs: "foo"`,
-			want: "into map[string]actionlint.ActionMetadataInputRequired",
+			want: "inputs must be mapping node",
 		},
 		{
 			what: "invalid inputs.*",
@@ -472,7 +520,7 @@ inputs:
 			what: "invalid outputs",
 			input: `name: Test
 outputs: "foo"`,
-			want: "into map[string]struct {}",
+			want: "outputs must be mapping node",
 		},
 	}
 
