@@ -13,6 +13,11 @@ type typedExpr struct {
 	pos Pos
 }
 
+func isExprAssigned(str *String) bool {
+	s := strings.TrimSpace(str.Value)
+	return strings.HasPrefix(s, "${{") && strings.HasSuffix(s, "}}")
+}
+
 // RuleExpression is a rule checker to check expression syntax in string values of workflow syntax.
 // It checks syntax and semantics of the expressions including type checks and functions/contexts
 // definitions. For more details see
@@ -90,24 +95,54 @@ func (rule *RuleExpression) VisitWorkflowPre(n *Workflow) error {
 			rule.checkStrings(e.Types, "")
 		case *WorkflowCallEvent:
 			ity := NewEmptyStrictObjectType()
-			for n, i := range e.Inputs {
+
+			// Set `inputs` context object before checking inputs since input's default values can refer `inputs` context.
+			//   inputs:
+			//     input1:
+			//       type: string
+			//     input2:
+			//       type: string
+			//       default: ${{ inputs.input1 }}
+			rule.inputsTy = ity
+
+			for _, i := range e.Inputs {
+				rule.checkString(i.Description, "")
+				// Check default value before setting type to `ity` because refering myself should cause an error.
+				//   inputs:
+				//     recursive:
+				//       type: string
+				//       default: ${{ inputs.recursive }}
+				ts := rule.checkString(i.Default, "on.workflow_call.inputs.<inputs_id>.default")
+
 				var ty ExprType
 				switch i.Type {
-				case WorkflowCallEventInputTypeBoolean:
-					ty = BoolType{}
 				case WorkflowCallEventInputTypeString:
 					ty = StringType{}
+				case WorkflowCallEventInputTypeBoolean:
+					ty = BoolType{}
+					if len(ts) == 1 && isExprAssigned(i.Default) {
+						switch ts[0].ty.(type) {
+						case BoolType, AnyType:
+							// ok
+						default:
+							rule.errorf(i.Default.Pos, "type of input %q must be bool but found type %s", i.Name.Value, ts[0].ty.String())
+						}
+					}
 				case WorkflowCallEventInputTypeNumber:
 					ty = NumberType{}
+					if len(ts) == 1 && isExprAssigned(i.Default) {
+						switch ts[0].ty.(type) {
+						case NumberType, AnyType:
+							// ok
+						default:
+							rule.errorf(i.Default.Pos, "type of input %q must be number but found type %s", i.Name.Value, ts[0].ty.String())
+						}
+					}
 				default:
 					ty = AnyType{}
 				}
-				ity.Props[n] = ty
-
-				rule.checkString(i.Description, "")
-				rule.checkString(i.Default, "on.workflow_call.inputs.<inputs_id>.default")
+				ity.Props[i.ID] = ty
 			}
-			rule.inputsTy = ity
 
 			// When no secret is passed, secrets may be inherited from a caller of the workflow.
 			// So `secrets` context must be typed as { string => string }. `e.Secrets` is nil when `secrets:` does not
@@ -542,7 +577,7 @@ func (rule *RuleExpression) checkWorkflowCall(c *WorkflowCall) {
 				}
 			}
 		case 1:
-			if strings.HasPrefix(v, "${{") && strings.HasSuffix(v, "}}") {
+			if isExprAssigned(i.Value) {
 				ty = ts[0].ty
 			}
 		}
@@ -609,8 +644,7 @@ func (rule *RuleExpression) checkIfCondition(str *String, workflowKey string) {
 		ts := rule.checkString(str, workflowKey)
 
 		if len(ts) == 1 {
-			s := strings.TrimSpace(str.Value)
-			if strings.HasPrefix(s, "${{") && strings.HasSuffix(s, "}}") {
+			if isExprAssigned(str) {
 				condTy = ts[0].ty
 			}
 		}
